% program to construct the post break test as described in Elliott and
% Mueller (2013) Section 4.


function [pbtstat,cv]=postbreaktest(th1,vth1,th2,vth2,gamm0)

% inputs
%         th1    100x1 vector with intial sample estimates 15-85
%         vth1   100x1 vector of variances of th1
%         th2    100x1 vector with post sample estimates 15-85
%         vth2   100x1 vector of variances of th2
%
%  
% numbers in first 14 and last 14 places are not used due to trimming
%
% outputs 
%         pbtstat   test statistic
%         cv        critical value
%         
% the cv changes depending on whether we switch for large SupF



% construct the SupF like statistic
bst=zeros(100,1);
for i=16:85;
    t1=th2(i,1)-th1(i-1,1);
    bst(i,1)=t1*t1/(vth2(i,1)+vth1(i-1,1));
end;
[sf,junk]=max(bst);

% construct estimate for the break date
l=(1:100)';
dprl=[th1(15,1)*ones(15,1);l(16:100,1).*th1(16:100)-l(15:99,1).*th1(15:99,1)];
dpsl=[0;(101-l(2:85,1)).*th2(1:84,1)-(100-l(2:85,1)).*th2(2:85,1);th2(85,1)*ones(15,1)];

lfn=ones(100,1)*999;
for i=17:85;
    t2a=dprl(16:i-1,1).*dprl(16:i-1,1);
    t2b=th1(i-1,1)*th1(i-1,1)*(i-1);

    t3a=dpsl(i+1:85,1).*dpsl(i+1:85,1);
    t3b=th2(i,1)*th2(i,1)*(100-i);
 
    lfn(i,1)=(sum(t2a)-t2b+sum(t3a)-t3b);
end;

[junk,lhat]=min(lfn);
if lhat==85; lhat=84; end;
if lhat<16; lhat=16; end;

% compute the post estimated break t stat
tpost=(th2(lhat+1,1)-gamm0)/sqrt(vth2(lhat+1,1));


% compute the S statistic
omh2=((lhat-1)^2/9900)*vth1(lhat-1,1)+(((100-lhat)^2)/9900)*vth2(lhat,1);
%omh2=(1-0.5*0.5)/150;
%omh2=vth1(49,1);
%[omh2 omh2a]
tparm=[
0.006	0.15	0.24	200	28;
0.038	0.75	0.85	200	28;
0.588	0.15	0.85	100	20;
0.067	0.85	0.85	4	3;
0.057	0.2	    0.74	300	16;
0.123	0.85	0.85	10	5;
0.008	0.15	0.19	10	5;
0.032	0.2	    0.74	10	9;
0.020	0.75	0.82	10	7;
0.002	0.83	0.84	10	13;
0.001	0.85	0.85	3	15.5;
0.026	0.2	    0.74	3	6;
0.001	0.75	0.82	3	13;
0.005	0.6	    0.69	10	12;
0.009	0.45	0.59	10	11;
0.004	0.8	    0.82	10	11;
0.009	0.7	    0.74	10	9;
0.004	0.6	    0.69	3	8];
    

% p     a      b      s2delt mudelt  


s2pre=378;
s2b=22;
s=((1:100)')/100;
th1c=th1-gamm0;

den=0;
for im=1:length(tparm);
    a=floor(tparm(im,2)*100);
    b=floor(tparm(im,3)*100);

    tr=[zeros(a-1,1);ones(b-a+1,1);zeros(100-b,1)];
    t0a=(tparm(im,1)/(b-a+1));
    d1=ones(100,1)+s*tparm(im,4);
    % compute peices inside exponentials in denominator
    t1=(tparm(im,5)/sqrt(omh2))*th1c.*s./d1;
    t2=-0.5*tparm(im,5)*tparm(im,5).*(s./d1);
    t3=(0.5/omh2)*tparm(im,4)*(th1c.*th1c.*s.*s./d1);
    den=den+t0a*(tr'*((exp(t2+t3)).*cosh(t1)./sqrt(d1)));



end;

v1=ones(100,1)+s*s2pre;
v2=ones(100,1)+(1-s)*s2b;
tmn=[zeros(14,1);ones(85-15+1,1);zeros(15,1)];
n1=s2pre*(th1c).*(th1c).*s.*s./(v1*omh2*2);
n2=s2b*(th2-gamm0).*(th2-gamm0).*(1-s).*(1-s)./(v2*omh2*2);
num=(1/71)*tmn'*(exp(n1+n2)./sqrt(v2.*v1));

lrhat=num/den;


if sf>90;
    pbtstat=abs(tpost);
    cv=2.01;
else;
    pbtstat=lrhat;
    cv=2.41;
    
end;
 



